#include "ajedrez.h"

using namespace ajedrez;

Pieza::Pieza(const string& nombre):tipo(nombre)
{
}

void Pieza::dibuja_pieza()
{
	cout << "Pieza::dibuja_pieza()" << endl;
}
	
void Pieza::set_posicion(int fila, int columna)
{
	cout << "Pieza::set_posicion()" << endl;
	posicion.fila = fila;
	posicion.columna = columna;
}
	
const Jugada& Pieza::get_posicion()
{
	return posicion;
}

const string& Pieza::get_tipo()
{
	return tipo;
}

Jugador::~Jugador()
{
	destruye_piezas();
}

void Jugador::mueve_pieza(int fila, int columna, Pieza& p)
{
	cout << "Jugador::mueve_pieza()" << endl;
	p.set_posicion(fila, columna);
	p.dibuja_pieza();
}

// Obtiene la pieza que ocupa la posicin fila,columna
Pieza* Jugador::get_contenido(int fila, int columna)
{
	cout << "Jugador::get_contenido()" << endl;
	
	map<string, Pieza*>::iterator it;

	for(it = mapa_piezas.begin(); it != mapa_piezas.end(); it++)
		if (((*it).second->get_posicion().fila == fila)
			&& ((*it).second->get_posicion().columna == columna))
			return (*it).second;
	
	return NULL;
}

map<string, Pieza*>* Jugador::get_mapa_piezas()
{
	cout << "Jugador::get_mapa_piezas()" << endl;	
	return NULL;
}

void Jugador::destruye_piezas()
{
	vector<Pieza*>::iterator it;

	for (it = piezas.begin(); it != piezas.end(); it++)
		delete *it;
}

bool Jugador::elimina_pieza(Pieza* pieza)
{
	cout << "Jugador::elimina_pieza()" << endl;
	
	vector<Pieza*>::iterator it = find(piezas.begin(), piezas.end(), pieza);

	if (it != piezas.end())
	{
		piezas.erase(it);
		return true;
	} else return false;
}

Jugador_humano::Jugador_humano(I_comunicacion& com):com_ref(com)
{
	Pieza* torre2 = new Pieza("Torre2");
	torre2->set_posicion(2,4);
	piezas.push_back(torre2);
	mapa_piezas["Torre2"] = torre2;

	Pieza* peon4 = new Pieza("Peon4");
	peon4->set_posicion(5,3);
	piezas.push_back(peon4);
	mapa_piezas["Peon4"] = peon4;
		
	Pieza* rey = new Pieza("Rey");
	rey->set_posicion(1,2);
	piezas.push_back(rey);
	mapa_piezas["Rey"] = rey;

}


void Jugador_humano::registrarse(const string& nombre, const string& password)
{
	cout << "Jugador::registrarse()" << endl;
	this->nombre = nombre;
	this->password = password;
}
	
int Jugador_humano::get_ELO()
{
	cout << "Jugador::get_ELO()" << endl;
	return 10;
}

IA::IA(Jugador_humano& humano)
{
	Pieza* caballo1 = new Pieza("Caballo1");
	caballo1->set_posicion(3,1);
	piezas.push_back(caballo1);
	mapa_piezas["Caballo1"] = caballo1;
}


void IA::set_algoritmo(Sistema_IA* sistema)
{
	algoritmo_agr = sistema;
}

Pieza* IA::pensar_jugada(int fila, int columna, string p)
{
	cout << "IA::pensar_jugada()" << endl;
	return algoritmo_agr->calcula_jugada(fila, columna, p);
}
	
Pieza* IA::aconsejar()
{
	cout << "IA::aconsejar()" << endl;
	return algoritmo_agr->aconsejar();
}

float Final::get_puntuacion()
{
	cout << "Final::get_puntuacion()" << endl;
	return 4.3f;
}

Sistema_IA::Sistema_IA(Jugador_humano& humano, IA& ia, Heuristica& heu):humano_ref(humano), ia_ref(ia), heu_ref(heu)
{
}

Pieza* Algoritmo_secuencial::aconsejar()
{
	return NULL;
}

Pieza* Algoritmo_secuencial::calcula_jugada(int fila, int columna, const string& p)
{
	return NULL;
}

Algoritmo_GPU::Algoritmo_GPU(Jugador_humano& humano, IA& ia, Heuristica& heu):Sistema_IA(humano, ia, heu)
{
}

Pieza* Algoritmo_GPU::aconsejar()
{
	cout << "Algoritmo_GPU::aconsejar()" << endl; 
	
	return humano_ref.get_contenido(5,3);
}

Pieza* Algoritmo_GPU::calcula_jugada(int fila, int columna, const string& p)
{
	cout << "Algoritmo_GPU::calcula_jugada()" << endl;
	
	map<string, Pieza*>* mapa = humano_ref.get_mapa_piezas();
	
	if (heu_ref.get_puntuacion() > 1.3f)
	{
		// Procesa en paralelo
		Pieza* pieza = ia_ref.get_contenido(3,1);
		// Procesa en paralelo
		pieza->set_posicion(1,2);
		return pieza;
	}
		
	return NULL;
}

void Tablero::notifica_mensaje(const string& mensaje)
{
	cout << "Tablero::notifica_mensaje(): " << mensaje << endl;
}

void Tablero::dibuja_tablero()
{
	cout << "Tablero::dibuja_tablero()" << endl;
}