#ifndef ajedrez_h
#define ajedrez_h

#include "../comunicaciones/comunicaciones.h"

#include <vector>
#include <map>
#include <string>
#include <iostream>
#include <algorithm>

using namespace std;
using namespace comunicaciones;

namespace ajedrez
{		
	class Jugada
	{
	public:
		int fila;
		int columna;
	};
	
	class Pieza
	{
	private:
		string estado;
		Jugada posicion;
		string tipo;
	public:
		Pieza(const string& nombre);
		void dibuja_pieza();
		void set_posicion(int fila, int columna);
		const Jugada& get_posicion();
		const string& get_tipo();
	};
	
	class Jugador
	{
	protected:
		int color;
		map<string, Pieza*> mapa_piezas;
		vector<Pieza*> piezas;
	public:
		~Jugador();
		void mueve_pieza(int fila, int columna, Pieza& p);
		Pieza* get_contenido(int fila, int columna);
		map<string, Pieza*>* get_mapa_piezas();
		bool elimina_pieza(Pieza* pieza);
	private:
		void destruye_piezas();
	};

	class Jugador_humano : public Jugador
	{
	private:
		string nombre;
		string password;
		I_comunicacion& com_ref;
	public:
		Jugador_humano(I_comunicacion& com);
		void registrarse(const string& nombre, const string& password);
		int get_ELO();
	};

	class Sistema_IA;
	
	class IA : public Jugador
	{
	private:
		int nivel;
		Sistema_IA* algoritmo_agr;
	public:
		IA(Jugador_humano& humano);
		Pieza* pensar_jugada(int fila, int columna, string p);
		Pieza* aconsejar();
		void set_algoritmo(Sistema_IA* sistema);
	};

	class Heuristica
	{
	private:
		float puntuacion;
	public:
		virtual float get_puntuacion() = 0;
	};

	class Final : public Heuristica
	{
	public:
		float get_puntuacion();
	};

	class Sistema_IA
	{
	protected:
		Jugador_humano& humano_ref;
		IA& ia_ref;
		Heuristica& heu_ref;
	public:	
		Sistema_IA(Jugador_humano& humano, IA& ia, Heuristica& heu);
		virtual Pieza* aconsejar() = 0;
		virtual Pieza* calcula_jugada(int fila, int columna, const string& p) = 0;
	};

	class Algoritmo_secuencial : public Sistema_IA
	{
	public:	
		Pieza* aconsejar();
		Pieza* calcula_jugada(int fila, int columna, const string& p);
	};

	class Algoritmo_GPU : public Sistema_IA
	{
	public:	
		Algoritmo_GPU(Jugador_humano& humano, IA& ia, Heuristica& heu);
		Pieza* aconsejar();
		Pieza* calcula_jugada(int fila, int columna, const string& p);
	};

	// Interfaz I_tablero
	class I_tablero
	{
	public:	
		virtual void dibuja_tablero() = 0;
	};

	// Implementa interfaces I_observer e I_tablero
	class Tablero : public I_observer, public I_tablero
	{
	public:
		void notifica_mensaje(const string& mensaje);
		void dibuja_tablero();
	};

}

#endif