#include "gestion_juego.h"

using namespace gestion_juego;

Control_juego::Control_juego(GUI::Gui& gui, comunicaciones::I_comunicacion& com, ajedrez::Jugador_humano& humano, ajedrez::IA& ia):gui_ref(gui), com_ref(com), humano_ref(humano), ia_ref(ia)
{
}

void Control_juego::on_origen(int x_orig, int y_orig)
{
	cout << "Control_juego::on_origen()" << endl;
	cout << "x_orig: " << x_orig << endl;
	cout << "y_orig: " << y_orig << endl;
	// Realiza conversion coordenadas de pantalla a filas y columnas
	fil_orig = convierte_xy(y_orig);
	col_orig = convierte_xy(x_orig);

}

void Control_juego::on_destino(int x_dest, int y_dest)
{
	cout << "Control_juego::on_destino()" << endl;
	cout << "x_dest: " << x_dest << endl;
	cout << "y_dest: " << y_dest << endl;
	fil_dest = convierte_xy(y_dest);
	col_dest = convierte_xy(x_dest);
	hacer_movimiento(fil_orig, col_orig, fil_dest, col_dest);
}


void Control_juego::on_consejo()
{
	cout << "Control_juego::on_consejo()" << endl;
	aconsejar();
}

void Control_juego::notifica_mensaje(const string& mensaje)
{
	cout << "Control_juego::notifica_mensaje(): " << mensaje << endl;
}

void Control_juego::aconsejar()
{
	cout << "Control_juego::aconsejar()" << endl;
	cout << "Aconsejando" << endl;
	cout << "Debe mover la pieza: " << ia_ref.aconsejar()->get_tipo() << endl;
}

void Control_juego::hacer_movimiento(int fil_orig, int col_orig, int fil_dest, int col_dest)
{
	cout << "Control_juego::hacer_movimiento()" << endl;
	cout << "haciendo movimiento desde " << fil_orig << "," << col_orig << " a " << fil_dest << "," << col_dest << endl;
	Pieza* pieza = humano_ref.get_contenido(fil_orig,col_orig);
	if (jugada_correcta(fil_dest, col_dest, pieza->get_tipo()))
	{
		humano_ref.mueve_pieza(fil_dest, col_dest, *pieza);
		gui_ref.dibuja_tablero();

		if (mate(fil_dest, col_dest))
		{
			pieza = ia_ref.get_contenido(fil_dest, col_dest);
			cout << "Se ha dado mate a pieza: " << pieza->get_tipo() << endl;
		}
		else if (detecta_amenaza(fil_dest, col_dest))
			gui_ref.alerta();
	}

}

void Control_juego::inicia_partida()
{
	cout << "Control_juego::iniciando_partida()" << endl;
}

void Control_juego::termina_partida()
{
	cout << "Control_juego::termina_partida()" << endl;
}
void Control_juego::buscar_contrincante()
{
	cout << "Control_juego::buscando_contrincante()" << endl;
}

void Control_juego::turno_ia()
{
	cout << "Control_juego::turno_ia()" << endl;
	
	Pieza* pieza = ia_ref.pensar_jugada(fil_dest,col_dest,"Torre2");
	cout << "IA ha decidio mover la pieza: " << pieza->get_tipo() << endl; 
	fil_dest = pieza->get_posicion().fila;
	col_dest = pieza->get_posicion().columna;

	if (jugada_correcta(fil_dest, col_dest, pieza->get_tipo()))
	{
		ia_ref.mueve_pieza(fil_dest, col_dest, *pieza);
		pieza = humano_ref.get_contenido(fil_dest, col_dest);
		if (detecta_amenaza(fil_dest, col_dest))
			cout << "La pieza de la IA est siendo amenazada" << endl;
		if (mate(fil_dest, col_dest))
		{
			cout << "Jaque al " << pieza->get_tipo() << endl;
			
			if (humano_ref.elimina_pieza(pieza))
				delete pieza;
				
			termina_partida();
		}
	}

}

bool Control_juego::detecta_amenaza(int fila, int columna)
{
	cout << "Control_juego::detectando amenaza()" << endl;
	return (((fila == 7) && (columna == 4)) ? true : false);
}

bool Control_juego::jugada_correcta(int fila, int columna, const string& pieza)
{
	cout << "Control_juego::jugada correcta()" << endl;
	return true;
}

bool Control_juego::mate(int fila, int columna)
{
	cout << "Control_juego::mate()" << endl;
	return (((fila == 7) && (columna == 4)) ? false : true);
}

int Control_juego::convierte_xy(int cord)
{
	cout << "Control_juego::convierte_xy()" << endl;
	return cord / 100;
}