#include "estados.h"

using namespace std;

Estado::Estado()
{
}

void Estado::iniciar(Driver* driver)
{
	check_error("EstInicio");
}

void Estado::pasan_dos_segundos(Driver* driver)
{
	check_error("EstComprobando");
}

void Estado::dato_recibido(Driver* driver)
{
	check_error("EstComprobando");
}

void Estado::error_lectura(Driver* driver)
{
	check_error("EstLeyendo");
}

void Estado::lectura_OK(Driver* driver)
{
	check_error("EstLeyendo");
}

void Estado::buffer_lleno(Driver* driver)
{
	check_error("EstAlmacenando");
}

void Estado::queda_espacio_buffer(Driver* driver)
{
	check_error("EstAlmacenando");
}

void Estado::pasan_diez_segundos(Driver* driver)
{
	check_error("EstDetenido");
}

void Estado::imprime_estado(Driver* driver)
{
	cout << nombre_estado << endl;
}

void Estado::check_error(const string& supuesto)
{
	if (nombre_estado != supuesto)
	{
		cout << "Error estado incorrecto" << endl;
	}
}

void Estado::cambia_estado(Driver* driver, Estado* estado)
{
	driver->cambia_estado(estado);
}

Est_inicio::Est_inicio() 
{
	nombre_estado = "EstInicio";
}

Estado* Est_inicio::instancia = NULL;

void Est_inicio::iniciar(Driver* driver)
{
	cout << "iniciar()" << endl;
	cambia_estado(driver, Est_comprobando::get_instancia());
}

Estado* Est_inicio::get_instancia()
{
	if (instancia == NULL)
	{
		return (instancia = new Est_inicio());
	} else return instancia;
}

Estado* Est_comprobando::instancia = NULL;

Est_comprobando::Est_comprobando()
{
	nombre_estado = "EstComprobando";
}

void Est_comprobando::pasan_dos_segundos(Driver* driver)
{
	cout << "pasan_dos_segundos()" << endl;
}

void Est_comprobando::dato_recibido(Driver* driver)
{
	cout << "dato_recibido()" << endl;
	cambia_estado(driver, Est_leyendo::get_instancia());
}

Estado* Est_comprobando::get_instancia()
{
	if (instancia == NULL) 
	{	
		return (instancia = new Est_comprobando());
	} else return instancia;
}

Estado* Est_leyendo::instancia = NULL;

Est_leyendo::Est_leyendo()
{
	nombre_estado = "EstLeyendo";
}

void Est_leyendo::error_lectura(Driver* driver)
{
	cout << "error_lectura()" << endl;
	cambia_estado(driver, Est_detenido::get_instancia());
}

void Est_leyendo::lectura_OK(Driver* driver)
{
	cout << "lectura_OK()" << endl;
	cambia_estado(driver, Est_almacenando::get_instancia());
}

Estado* Est_leyendo::get_instancia()
{
	if (instancia == NULL) 
	{
		return (instancia = new Est_leyendo());
	} else return instancia;

}

Estado* Est_almacenando::instancia = NULL;

Est_almacenando::Est_almacenando()
{
	nombre_estado = "EstAlmacenando";
}

void Est_almacenando::buffer_lleno(Driver* driver)
{
	cout << "buffer_lleno()" << endl;
	cambia_estado(driver, Est_detenido::get_instancia());

}

void Est_almacenando::queda_espacio_buffer(Driver* driver)
{
	cout << "queda_espacio_buffer()" << endl;
	cambia_estado(driver, Est_final::get_instancia());
}

Estado* Est_almacenando::get_instancia()
{
	if (instancia == NULL) 
	{
		return (instancia = new Est_almacenando());
	} else	return instancia;
}

Estado* Est_detenido::instancia = NULL;

Est_detenido::Est_detenido()
{
	nombre_estado = "EstDetenido";
}

void Est_detenido::pasan_diez_segundos(Driver* driver)
{
	cout << "pasan_diez_segundos()" << endl;
	cambia_estado(driver, Est_final::get_instancia());
}

Estado* Est_detenido::get_instancia()
{
	if (instancia == NULL) 
	{
		return (instancia = new Est_detenido());
	} else return instancia;
}

Estado* Est_final::instancia = NULL;

Est_final::Est_final()
{
	nombre_estado = "EstFinal";
}

Estado* Est_final::get_instancia()
{
	if (instancia == NULL)
	{
		return (instancia = new Est_final());
	} else return instancia;
}

Driver::Driver()
{
	// Estado inicial
	estado_actual = Est_inicio::get_instancia();
}

void Driver::iniciar()
{
	estado_actual->iniciar(this);
}

void Driver::pasan_dos_segundos()
{
	estado_actual->pasan_dos_segundos(this);
}

void Driver::dato_recibido()
{
	estado_actual->dato_recibido(this);
	//inhibe_interrupciones();
}

void Driver::error_lectura()
{
	estado_actual->error_lectura(this);
	//activa_interrupciones();
}

void Driver::lectura_OK()
{
	estado_actual->lectura_OK(this);
}

void Driver::buffer_lleno()
{
	estado_actual->buffer_lleno(this);
	//activa_interrupciones();
}

void Driver::queda_espacio_buffer()
{
	estado_actual->queda_espacio_buffer(this);
	//activa_interrupciones();
}

void Driver::pasan_diez_segundos()
{
	estado_actual->pasan_diez_segundos(this);
}

void Driver::imprime_estado()
{
	estado_actual->imprime_estado(this);
}

void Driver::cambia_estado(Estado* estado)
{
	estado_actual = estado;
}

int main()
{
	_CrtSetDbgFlag ( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
	
	Driver* driver = new Driver();
	
	// Comienza transiciones	
	driver->imprime_estado();
	driver->iniciar();
	driver->imprime_estado();

	driver->imprime_estado();
	driver->pasan_dos_segundos();
	driver->imprime_estado();

	driver->imprime_estado();
	driver->pasan_dos_segundos();
	driver->imprime_estado();

	driver->imprime_estado();
	driver->dato_recibido();
	driver->imprime_estado();

	/*driver->imprime_estado();
	driver->lectura_OK();
	driver->imprime_estado();

	driver->imprime_estado();
	driver->queda_espacio_buffer();
	driver->imprime_estado();*/ 

	driver->imprime_estado();
	driver->error_lectura();
	driver->imprime_estado();
	
	driver->imprime_estado();
	driver->pasan_diez_segundos();
	driver->imprime_estado();
	
	delete Est_inicio::get_instancia();
	delete Est_comprobando::get_instancia();
	delete Est_leyendo::get_instancia();
	delete Est_almacenando::get_instancia();
	delete Est_detenido::get_instancia();
	delete Est_final::get_instancia();

	delete driver;

	cin.get();
}

