import abc

#Clase abstracta para gestión del estado
class Estado(metaclass = abc.ABCMeta):
    @property
    def driver(self):
        return self._driver

    @driver.setter
    def context(self, driver):
        self._driver = driver

    def iniciar(self):
        self.check_error("EstInicio")
    
    def pasan_dos_segundos(self):
        self.check_error("EstComprobando")
    
    def dato_recibido(self):
        self.check_error("EstComprobando")
        
    def error_lectura(self):
        self.check_error("EstLeyendo")
    
    def lectura_OK(self):
        self.check_error("EstLeyendo")
        
    def buffer_lleno(self):
        self.check_error("EstAlmacenando")
    
    def queda_espacio_buffer(self):
        self.check_error("EstAlmacenando")
        
    def pasan_diez_segundos(self):
        self.check_error("EstDetenido")
    
    def imprime_estado(self):
        print(f"{self.nombre_estado}")
        
    # Comprueba si en estado correcto
    def check_error(self, supuesto):
        if (self.nombre_estado != supuesto):
            print("Error estado incorrecto")
        
# Estado de inicio    
class EstInicio(Estado):
    def __init__(self):
        self.nombre_estado = "EstInicio"
        
    def iniciar(self):
        print("iniciar()")
        self._driver.cambia_estado(EstComprobando())
        
# Estado comprobando
class EstComprobando(Estado):
    def __init__(self):
        self.nombre_estado = "EstComprobando"
    
    def pasan_dos_segundos(self):
        print("pasan_dos_segundos()")
        self._driver.cambia_estado(EstComprobando())
        
    def dato_recibido(self):
        print("dato_recibido()")
        self._driver.cambia_estado(EstLeyendo())

#Estado leyendo
class EstLeyendo(Estado):
    def __init__(self):
        self.nombre_estado = "EstLeyendo"
   
    def error_lectura(self):
        print("error_lectura()")
        self._driver.cambia_estado(EstDetenido())
        
    def lectura_OK(self):
        print("lecturaOk()")
        self._driver.cambia_estado(EstAlmacenando())
      
#Estado almacenando
class EstAlmacenando(Estado):
    def __init__(self):
        self.nombre_estado = "EstAlmacenando"
    
    def buffer_lleno(self):
        print("buffer_lleno()")
        self._driver.cambia_estado(EstDetenido())
    
    def queda_espacio_buffer(self):
        print("queda_espacio_buffer()")
        self._driver.cambia_estado(EstFinal())
        
#Estado detenido
class EstDetenido(Estado):
    def __init__(self):
        self.nombre_estado = "EstDetenido"
        
    def pasan_diez_segundos(self):
        print("pasan_diez_segundos()")
        self._driver.cambia_estado(EstFinal())

#Estado final
class EstFinal(Estado): 
    def __init__(self):
        self.nombre_estado = "EstFinal"
               

# Clase contexto
class Driver:
    __estado_actual = None
    
    def __init__(self, estado):
        self.cambia_estado(estado)
    
    def cambia_estado(self, estado):
        Driver.__estado_actual = estado
        Driver.__estado_actual._driver = self
        
    def iniciar(self):
        Driver.__estado_actual.iniciar()
    
    def pasan_dos_segundos(self):
        Driver.__estado_actual.pasan_dos_segundos()
     
    def dato_recibido(self):
        Driver.__estado_actual.dato_recibido()
        #inhibe_interrupciones()
                
    def error_lectura(self):
        Driver.__estado_actual.error_lectura()
        #activa_interrupciones()
        
    def lectura_OK(self):
        Driver.__estado_actual.lectura_OK()
        
    def buffer_lleno(self):
        Driver.__estado_actual.buffer_lleno()    
        #activa_interrupciones()
        
    def queda_espacio_buffer(self):
        Driver.__estado_actual.queda_espacio_buffer()
        #activa_interrupciones
    
    def pasan_diez_segundos(self):
        Driver.__estado_actual.pasan_diez_segundos()
        
    def imprime_estado(self):
        Driver.__estado_actual.imprime_estado()
        
# Empieza transiciones        
driver = Driver(EstInicio())    
driver.imprime_estado()
driver.iniciar() 
driver.imprime_estado()

driver.imprime_estado()
driver.pasan_dos_segundos()  
driver.imprime_estado()

driver.imprime_estado()
driver.pasan_dos_segundos()
driver.imprime_estado()

driver.imprime_estado()
driver.dato_recibido()
driver.imprime_estado()

driver.imprime_estado()
driver.lectura_OK()
driver.imprime_estado()

driver.imprime_estado()
driver.buffer_lleno()
driver.imprime_estado()

driver.imprime_estado()
driver.pasan_diez_segundos() 
driver.imprime_estado()