import abc
from enum import Enum
from cifrador.cifrador import CifradorSimetrico, CifradorAsimetrico
from comunicaciones.comunicaciones import FachadaComunicaciones
            
# Enumeración de opciones      
class Opcion(Enum):
    SIMETRICO = 1
    HIBRIDO = 2

# Interfaz de Fachada Menu
class IMenuCifrado(metaclass = abc.ABCMeta):
    @abc.abstractmethod
    def set_opcion(self, opcion):
        pass
     

# Clase para el usuario normal
class Usuario(IMenuCifrado):
    def __init__(self, nombre, login, password):
        self.__simetrico = None
        self.__asimetrico = None
        self.__numero_accesos = 0
        self.__nombre = nombre
        self._login = login
        self._password = password     
        # Instancia la Fachada de Comunicaciones
        self._conexion = FachadaComunicaciones()
        
    # Envía datos cifrados
    def __envia_datos(self, msg, clave, nonce, tag):
        try:
            self._conexion.conecta()
        except:
            print("Error de conexion")  
            return    
        try:
            self._conexion.envia_msg(msg)
            self._conexion.envia_clave(clave)
            self._conexion.envia_nonce(nonce)
            self._conexion.envia_tag(tag)
        except:
            print("Error enviando datos por socket")    
        finally:    
            self._conexion.desconecta()       
       
    def _introduce_mensaje(self):
        return input("Introduzca el mensaje a cifrar: ")
        
    # Genera clave de 16 bytes automáticamente 
    def _introduce_clave(self):
        clave = CifradorSimetrico.genera_clave()
        print (f"Introduzca la clave de cifrado: {clave}")
        return clave    
        
    # Cifra con algoritmo simétrico AES
    def cifra_simetrico(self, msg, clave):
        if self.__simetrico == None:
               print("Creando cifrador simétrico...")
               self.__simetrico = CifradorSimetrico()
        
        self.__numero_accesos = self.__numero_accesos + 1
        self.__simetrico.cifra(msg, clave)
        msg, nonce, tag = self.__simetrico.get_msg()
  
        print (f"Mensaje cifrado= {msg}")
        self.__envia_datos(msg, clave, nonce, tag)
         
    # Cifra con algoritmo híbrido AES-RSA
    def cifra_hibrido(self, msg):
        if self.__asimetrico == None:
            print("Creando cifrador asimétrico...")
            self.__asimetrico = CifradorAsimetrico()
            if self.__simetrico == None:
                print("Creando cifrador simétrico...")
                self.__simetrico = CifradorSimetrico()
            print("Por favor, espere...")
            self.__asimetrico.crea_claves()
            self.__clave_sesion = self.__simetrico.genera_clave()
            self.__asimetrico.cifra(self.__clave_sesion)
            self.clave_sesion_cifrada = self.__asimetrico.get_msg()   
        
        self.__numero_accesos = self.__numero_accesos + 1
        self.__simetrico.cifra(msg,self.__clave_sesion)
        msg, nonce, tag = self.__simetrico.get_msg()     
        
        print (f"Mensaje cifrado= {msg}")
        self.__envia_datos(msg, self.clave_sesion_cifrada, nonce, tag)
        
    # Define opción
    def set_opcion(self, opcion):
        if opcion == Opcion.SIMETRICO:   
            msg = self._introduce_mensaje()
            clave = self._introduce_clave()
            self.cifra_simetrico(msg, clave)
        elif opcion == Opcion.HIBRIDO:
             msg = self._introduce_mensaje()
             self.cifra_hibrido(msg)   
             
             