import abc
from Crypto.Random import get_random_bytes
from Crypto.Cipher import AES, PKCS1_OAEP
from Crypto.PublicKey import RSA

# Patrón Strategy. Interfaz Cifrador
class Cifrador(metaclass = abc.ABCMeta):
    @abc.abstractmethod
    def cifra(self, msg, clave):
        pass
        
# Implementa cifrador simétrico        
class CifradorSimetrico(Cifrador):      
    @staticmethod
    def genera_clave():
        return get_random_bytes(16)
     
    def cifra(self, msg, clave):
        cifrador = AES.new(clave, AES.MODE_EAX)
        self.__nonce = cifrador.nonce
        self.__msg_cifrado, self.__tag = cifrador.encrypt_and_digest(msg.encode("UTF-8"))
    # Devuelve datos necesarios de cifrado  
    def get_msg(self):
        return self.__msg_cifrado, self.__nonce, self.__tag
 
# Implementa cifrador híbrido
class CifradorAsimetrico(Cifrador):
    def cifra(self, msg, clave = None):
          # Encripta la clave de sesión con la clave pública RSA
        clave_publica = RSA.import_key(open("publica.pem").read())
        cifrador_rsa = PKCS1_OAEP.new(clave_publica)
        self.__clave_sesion_cifrada = cifrador_rsa.encrypt(msg)    
    
    # Crea par de claves publica y privada
    def crea_claves(self):
        clave = RSA.generate(2048)
        clave_publica = clave.publickey().exportKey()
        with open("publica.pem", "wb") as f:
            f.write(clave_publica)    
        clave_privada = clave.exportKey()
        with open("privada.pem", "wb") as f:
            f.write(clave_privada)
 
    # Devuelve datos necesarios de cifrado
    def get_msg(self):
        return self.__clave_sesion_cifrada
     
      

 