import abc
import socket
from .notificador import Notificador

# Interfaz IComunicación
class IComunicacion(metaclass = abc.ABCMeta):
    @abc.abstractmethod
    def conecta(self):
        pass
    
    @abc.abstractmethod
    def desconecta(self):
        pass

    @abc.abstractmethod
    def envia_msg(self, msg):
        pass
    
    @abc.abstractmethod
    def envia_clave(self, clave):
        pass  
   
    @abc.abstractmethod
    def envia_nonce(self, nonce):
        pass  
    
    @abc.abstractmethod
    def envia_tag(self, tag):
        pass      
    
    @abc.abstractmethod
    def adjunta(self, observer):
        pass
    
    @abc.abstractmethod
    def activa_servidor(self):
        pass
 
        
# Fachada de comunicaciones. Implementa IComunicación.        
class FachadaComunicaciones(IComunicacion):
    
    # Inicia Notificador
    def __init__(self):
        self.notificador = Notificador()
            
    # Conecta socket
    def conecta(self):
        # Crea el socket TCP/IP
        self.__sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        direccion_servidor = ("localhost", 10000)
        print(f"conectando a {direccion_servidor}")
        self.__sock.connect(direccion_servidor)
        
    # Desconecta socket
    def desconecta(self):
        print("cerrando conexion")
        self.__sock.close()
        
    # Envía mensaje cifrado    
    def envia_msg(self,msg):
        print ("enviando mensaje cifrado...")
        self.__sock.sendall(msg)
    
    # Envía clave    
    def envia_clave(self,clave):
        print("enviando clave sesión cifrada...")
        self.__sock.sendall(clave)    
    
    # Envía nonce
    def envia_nonce(self, nonce):
        print("enviando nonce...")
        self.__sock.sendall(nonce)
        
    # Envía tag
    def envia_tag(self, tag):
        print("enviando tag...")
        self.__sock.sendall(tag)
    
    # Adjunta observador
    def adjunta(self, observer):
        self.notificador.adjunta(observer)
    
    # Actia el servidor
    def activa_servidor(self):
        # Crea socket TCP/IP para el servidor
        self.__sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        direccion_servidor = ('localhost', 10000)
        print(f"Iniciando en {direccion_servidor}")
        
        try:
            self.__sock.bind(direccion_servidor)
            # Escucha conexiones entrantes
            self.__sock.listen(1) 
    
            # Espera conexión con cliente
            print("Esperando conexión...")
            conexion, direccion_cliente = self.__sock.accept()

            print (f"Conectado a: {direccion_cliente}")

            # Recibe datos
            mensaje_cifrado = conexion.recv(1024)
            clave = conexion.recv(1024)
            nonce = conexion.recv(1024)
            tag = conexion.recv(1024)
            self.notificador.notifica(mensaje_cifrado, clave, nonce, tag)         
        except:
            print("Error activando escucha en servidor (posiblemente exista otro)")
 
        finally:
            # Cierra el socket y la conexión
            self.__sock.close()
            print("Conexión cerrada")
