import abc
from Crypto.Cipher import AES, PKCS1_OAEP
from Crypto.PublicKey import RSA

# Patrón Strategy. Interfaz Descifrador.
class Descifrador(metaclass = abc.ABCMeta):
    @abc.abstractmethod
    def descifra(self, msg, clave, nonce, tag):
        pass
        
# Implementa descrifrador simétrico        
class DescifradorSimetrico(Descifrador):      
    def descifra(self, msg, clave, nonce, tag):    
        descifrador_aes = AES.new(clave, AES.MODE_EAX, nonce)
        mensaje_descifrado = descifrador_aes.decrypt_and_verify(msg,tag)
        self.__mensaje_descifrado = mensaje_descifrado.decode("UTF-8")
                   
    # Devuelve mensaje descifrado
    def get_msg(self):
        return self.__mensaje_descifrado
 
# Implementa descifrador asimétrico
class DescifradorAsimetrico(Descifrador):
    def descifra(self, msg, clave = None, nonce = None, tag = None):
        clave_privada = RSA.import_key(open("privada.pem").read())
    
        # Desencripta la clave de sesión con la clave privada RSA
        descifrador_rsa = PKCS1_OAEP.new(clave_privada)
        self.__mensaje_descifrado = descifrador_rsa.decrypt(msg) 

    # Devuelve mensaje descifrado
    def get_msg(self):
        return self.__mensaje_descifrado
     
      

 